"""
_WorkUnit.Delete_

Oracle implementation of WorkUnit.Delete
"""

from __future__ import absolute_import, division, print_function

from WMCore.WMBS.MySQL.WorkUnit.Delete import Delete as DeleteMySQL


class Delete(DeleteMySQL):
    """
    _WorkUnit.Delete_

    Oracle implementation of WorkUnit.Delete
    """

    pass
