#!/usr/bin/env python
"""
_KillWorkflow_

Oracle implementation of Jobs.KillWorkflow
"""

from WMCore.WMBS.MySQL.Jobs.KillWorkflow import KillWorkflow as KillWorkflowMySQL

class KillWorkflow(KillWorkflowMySQL):
    pass
